@@ -5,7 +5,7 @@
 
     Several HTML builders.
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -14,14 +14,11 @@ import sys
 import zlib
 import codecs
 import posixpath
-import cPickle as pickle
 from os import path
-try:
-    from hashlib import md5
-except ImportError:
-    # 2.4 compatibility
-    from md5 import md5
+from hashlib import md5
 
+from six import iteritems, text_type, string_types
+from six.moves import cPickle as pickle
 from docutils import nodes
 from docutils.io import DocTreeInput, StringOutput
 from docutils.core import Publisher
@@ -32,11 +29,9 @@ from docutils.readers.doctree import Reader as DoctreeReader
 from sphinx import package_dir, __version__
 from sphinx.util import jsonimpl, copy_static_entry
 from sphinx.util.osutil import SEP, os_path, relative_uri, ensuredir, \
-     movefile, ustrftime, copyfile
+    movefile, ustrftime, copyfile
 from sphinx.util.nodes import inline_all_toctrees
 from sphinx.util.matching import patmatch, compile_matchers
-from sphinx.util.pycompat import any, b
-from sphinx.errors import SphinxError
 from sphinx.locale import _
 from sphinx.search import js_index
 from sphinx.theming import Theme
@@ -45,7 +40,7 @@ from sphinx.application import ENV_PICKLE_FILENAME
 from sphinx.highlighting import PygmentsBridge
 from sphinx.util.console import bold, darkgreen, brown
 from sphinx.writers.html import HTMLWriter, HTMLTranslator, \
-     SmartyPantsHTMLTranslator
+    SmartyPantsHTMLTranslator
 
 #: the filename for the inventory of objects
 INVENTORY_FILENAME = 'objects.inv'
@@ -63,7 +58,7 @@ def get_stable_hash(obj):
         return get_stable_hash(list(obj.items()))
     elif isinstance(obj, (list, tuple)):
         obj = sorted(get_stable_hash(o) for o in obj)
-    return md5(unicode(obj).encode('utf8')).hexdigest()
+    return md5(text_type(obj).encode('utf8')).hexdigest()
 
 
 class StandaloneHTMLBuilder(Builder):
@@ -83,6 +78,7 @@ class StandaloneHTMLBuilder(Builder):
     searchindex_filename = 'searchindex.js'
     add_permalinks = True
     embedded = False  # for things like HTML help or Qt help: suppresses sidebar
+    search = True  # for things like HTML help and Apple help: suppress search
 
     # This is a class attribute because it is mutated by Sphinx.add_javascript.
     script_files = ['_static/jquery.js', '_static/underscore.js',
@@ -100,6 +96,8 @@ class StandaloneHTMLBuilder(Builder):
         # a hash of all config values that, if changed, cause a full rebuild
         self.config_hash = ''
         self.tags_hash = ''
+        # basename of images directory
+        self.imagedir = '_images'
         # section numbers for headings in the currently visited document
         self.secnumbers = {}
         # currently written docname
@@ -140,7 +138,7 @@ class StandaloneHTMLBuilder(Builder):
         Theme.init_themes(self.confdir, self.config.html_theme_path,
                           warn=self.warn)
         themename, themeoptions = self.get_theme_config()
-        self.theme = Theme(themename)
+        self.theme = Theme(themename, warn=self.warn)
         self.theme_options = themeoptions.copy()
         self.create_template_bridge()
         self.templates.init(self, self.theme)
@@ -157,7 +155,9 @@ class StandaloneHTMLBuilder(Builder):
                                           self.config.trim_doctest_flags)
 
     def init_translator_class(self):
-        if self.config.html_translator_class:
+        if self.translator_class is not None:
+            pass
+        elif self.config.html_translator_class:
             self.translator_class = self.app.import_object(
                 self.config.html_translator_class,
                 'html_translator_class setting')
@@ -168,7 +168,7 @@ class StandaloneHTMLBuilder(Builder):
 
     def get_outdated_docs(self):
         cfgdict = dict((name, self.config[name])
-                       for (name, desc) in self.config.values.iteritems()
+                       for (name, desc) in iteritems(self.config.values)
                        if desc[1] == 'html')
         self.config_hash = get_stable_hash(cfgdict)
         self.tags_hash = get_stable_hash(sorted(self.tags))
@@ -194,7 +194,7 @@ class StandaloneHTMLBuilder(Builder):
         except Exception:
             pass
         if old_config_hash != self.config_hash or \
-               old_tags_hash != self.tags_hash:
+           old_tags_hash != self.tags_hash:
             for docname in self.env.found_docs:
                 yield docname
             return
@@ -225,13 +225,13 @@ class StandaloneHTMLBuilder(Builder):
         """Utility: Render a lone doctree node."""
         if node is None:
             return {'fragment': ''}
-        doc = new_document(b('<partial node>'))
+        doc = new_document(b'<partial node>')
         doc.append(node)
 
         if self._publisher is None:
             self._publisher = Publisher(
-                    source_class = DocTreeInput,
-                    destination_class=StringOutput)
+                source_class = DocTreeInput,
+                destination_class=StringOutput)
             self._publisher.set_components('standalone',
                                            'restructuredtext', 'pseudoxml')
 
@@ -248,14 +248,16 @@ class StandaloneHTMLBuilder(Builder):
 
     def prepare_writing(self, docnames):
         # create the search indexer
-        from sphinx.search import IndexBuilder, languages
-        lang = self.config.html_search_language or self.config.language
-        if not lang or lang not in languages:
-            lang = 'en'
-        self.indexer = IndexBuilder(self.env, lang,
-                                    self.config.html_search_options,
-                                    self.config.html_search_scorer)
-        self.load_indexer(docnames)
+        self.indexer = None
+        if self.search:
+            from sphinx.search import IndexBuilder, languages
+            lang = self.config.html_search_language or self.config.language
+            if not lang or lang not in languages:
+                lang = 'en'
+            self.indexer = IndexBuilder(self.env, lang,
+                                        self.config.html_search_options,
+                                        self.config.html_search_scorer)
+            self.load_indexer(docnames)
 
         self.docwriter = HTMLWriter(self)
         self.docsettings = OptionParser(
@@ -269,7 +271,8 @@ class StandaloneHTMLBuilder(Builder):
         # html_domain_indices can be False/True or a list of index names
         indices_config = self.config.html_domain_indices
         if indices_config:
-            for domain in self.env.domains.itervalues():
+            for domain_name in sorted(self.env.domains):
+                domain = self.env.domains[domain_name]
                 for indexcls in domain.indices:
                     indexname = '%s-%s' % (domain.name, indexcls.name)
                     if isinstance(indices_config, list):
@@ -277,7 +280,7 @@ class StandaloneHTMLBuilder(Builder):
                             continue
                     # deprecated config value
                     if indexname == 'py-modindex' and \
-                           not self.config.html_use_modindex:
+                       not self.config.html_use_modindex:
                         continue
                     content, collapse = indexcls(domain).generate()
                     if content:
@@ -293,14 +296,14 @@ class StandaloneHTMLBuilder(Builder):
             self.last_updated = None
 
         logo = self.config.html_logo and \
-               path.basename(self.config.html_logo) or ''
+            path.basename(self.config.html_logo) or ''
 
         favicon = self.config.html_favicon and \
-                  path.basename(self.config.html_favicon) or ''
+            path.basename(self.config.html_favicon) or ''
         if favicon and os.path.splitext(favicon)[1] != '.ico':
             self.warn('html_favicon is not an .ico file')
 
-        if not isinstance(self.config.html_use_opensearch, basestring):
+        if not isinstance(self.config.html_use_opensearch, string_types):
             self.warn('html_use_opensearch config value must now be a string')
 
         self.relations = self.env.collect_relations()
@@ -338,6 +341,7 @@ class StandaloneHTMLBuilder(Builder):
             show_source = self.config.html_show_sourcelink,
             file_suffix = self.out_suffix,
             script_files = self.script_files,
+            language = self.config.language,
             css_files = self.css_files,
             sphinx_version = __version__,
             style = stylename,
@@ -350,7 +354,7 @@ class StandaloneHTMLBuilder(Builder):
         if self.theme:
             self.globalcontext.update(
                 ('theme_' + key, val) for (key, val) in
-                self.theme.get_options(self.theme_options).iteritems())
+                iteritems(self.theme.get_options(self.theme_options)))
         self.globalcontext.update(self.config.html_context)
 
     def get_doc_context(self, docname, body, metatags):
@@ -390,8 +394,9 @@ class StandaloneHTMLBuilder(Builder):
                 pass
             related = self.relations.get(related[0])
         if parents:
-            parents.pop() # remove link to the master file; we have a generic
-                          # "back to index" link already
+            # remove link to the master file; we have a generic
+            # "back to index" link already
+            parents.pop()
         parents.reverse()
 
         # title rendered as HTML
@@ -427,6 +432,7 @@ class StandaloneHTMLBuilder(Builder):
         doctree.settings = self.docsettings
 
         self.secnumbers = self.env.toc_secnumbers.get(docname, {})
+        self.fignumbers = self.env.toc_fignumbers.get(docname, {})
         self.imgpath = relative_uri(self.get_target_uri(docname), '_images')
         self.dlpath = relative_uri(self.get_target_uri(docname), '_downloads')
         self.current_docname = docname
@@ -439,19 +445,26 @@ class StandaloneHTMLBuilder(Builder):
         self.handle_page(docname, ctx, event_arg=doctree)
 
     def write_doc_serialized(self, docname, doctree):
-        self.imgpath = relative_uri(self.get_target_uri(docname), '_images')
+        self.imgpath = relative_uri(self.get_target_uri(docname), self.imagedir)
         self.post_process_images(doctree)
         title = self.env.longtitles.get(docname)
         title = title and self.render_partial(title)['title'] or ''
         self.index_page(docname, doctree, title)
 
     def finish(self):
-        self.info(bold('writing additional files...'), nonl=1)
+        self.finish_tasks.add_task(self.gen_indices)
+        self.finish_tasks.add_task(self.gen_additional_pages)
+        self.finish_tasks.add_task(self.copy_image_files)
+        self.finish_tasks.add_task(self.copy_download_files)
+        self.finish_tasks.add_task(self.copy_static_files)
+        self.finish_tasks.add_task(self.copy_extra_files)
+        self.finish_tasks.add_task(self.write_buildinfo)
 
-        # pages from extensions
-        for pagelist in self.app.emit('html-collect-pages'):
-            for pagename, context, template in pagelist:
-                self.handle_page(pagename, context, template)
+        # dump the search index
+        self.handle_finish()
+
+    def gen_indices(self):
+        self.info(bold('generating indices...'), nonl=1)
 
         # the global general index
         if self.get_builder_config('use_index', 'html'):
@@ -460,38 +473,40 @@ class StandaloneHTMLBuilder(Builder):
         # the global domain-specific indices
         self.write_domain_indices()
 
-        # the search page
-        if self.name != 'htmlhelp':
-            self.info(' search', nonl=1)
-            self.handle_page('search', {}, 'search.html')
+        self.info()
+
+    def gen_additional_pages(self):
+        # pages from extensions
+        for pagelist in self.app.emit('html-collect-pages'):
+            for pagename, context, template in pagelist:
+                self.handle_page(pagename, context, template)
+
+        self.info(bold('writing additional pages...'), nonl=1)
 
         # additional pages from conf.py
         for pagename, template in self.config.html_additional_pages.items():
             self.info(' '+pagename, nonl=1)
             self.handle_page(pagename, {}, template)
 
-        if self.config.html_use_opensearch and self.name != 'htmlhelp':
+        # the search page
+        if self.search:
+            self.info(' search', nonl=1)
+            self.handle_page('search', {}, 'search.html')
+
+        # the opensearch xml file
+        if self.config.html_use_opensearch and self.search:
             self.info(' opensearch', nonl=1)
             fn = path.join(self.outdir, '_static', 'opensearch.xml')
             self.handle_page('opensearch', {}, 'opensearch.xml', outfilename=fn)
 
         self.info()
 
-        self.copy_image_files()
-        self.copy_download_files()
-        self.copy_static_files()
-        self.copy_extra_files()
-        self.write_buildinfo()
-
-        # dump the search index
-        self.handle_finish()
-
     def write_genindex(self):
         # the total count of lines for each index letter, used to distribute
         # the entries into two columns
         genindex = self.env.create_index(self)
         indexcounts = []
-        for _, entries in genindex:
+        for _k, entries in genindex:
             indexcounts.append(sum(1 + len(subitems)
                                    for _, (_, subitems) in entries))
 
@@ -528,14 +543,14 @@ class StandaloneHTMLBuilder(Builder):
     def copy_image_files(self):
         # copy image files
         if self.images:
-            ensuredir(path.join(self.outdir, '_images'))
-            for src in self.status_iterator(self.images, 'copying images... ',
-                                            brown, len(self.images)):
+            ensuredir(path.join(self.outdir, self.imagedir))
+            for src in self.app.status_iterator(self.images, 'copying images... ',
+                                                brown, len(self.images)):
                 dest = self.images[src]
                 try:
                     copyfile(path.join(self.srcdir, src),
-                             path.join(self.outdir, '_images', dest))
-                except Exception, err:
+                             path.join(self.outdir, self.imagedir, dest))
+                except Exception as err:
                     self.warn('cannot copy image file %r: %s' %
                               (path.join(self.srcdir, src), err))
 
@@ -543,14 +558,14 @@ class StandaloneHTMLBuilder(Builder):
         # copy downloadable files
         if self.env.dlfiles:
             ensuredir(path.join(self.outdir, '_downloads'))
-            for src in self.status_iterator(self.env.dlfiles,
-                                            'copying downloadable files... ',
-                                            brown, len(self.env.dlfiles)):
+            for src in self.app.status_iterator(self.env.dlfiles,
+                                                'copying downloadable files... ',
+                                                brown, len(self.env.dlfiles)):
                 dest = self.env.dlfiles[src][1]
                 try:
                     copyfile(path.join(self.srcdir, src),
                              path.join(self.outdir, '_downloads', dest))
-                except Exception, err:
+                except Exception as err:
                     self.warn('cannot copy downloadable file %r: %s' %
                               (path.join(self.srcdir, src), err))
 
@@ -569,9 +584,11 @@ class StandaloneHTMLBuilder(Builder):
                 copyfile(jsfile, path.join(self.outdir, '_static',
                                            'translations.js'))
 
-        # add context items for search function used in searchtools.js_t
         ctx = self.globalcontext.copy()
-        ctx.update(self.indexer.context_for_searchtool())
+
+        # add context items for search function used in searchtools.js_t
+        if self.indexer is not None:
+            ctx.update(self.indexer.context_for_searchtool())
 
         # then, copy over theme-supplied static files
         if self.theme:
@@ -583,10 +600,7 @@ class StandaloneHTMLBuilder(Builder):
         # then, copy over all user-supplied static files
         staticentries = [path.join(self.confdir, spath)
                          for spath in self.config.html_static_path]
-        matchers = compile_matchers(
-            self.config.exclude_patterns +
-            ['**/' + d for d in self.config.exclude_dirnames]
-        )
+        matchers = compile_matchers(self.config.exclude_patterns)
         for entry in staticentries:
             if not path.exists(entry):
                 self.warn('html_static_path entry %r does not exist' % entry)
@@ -597,15 +611,17 @@ class StandaloneHTMLBuilder(Builder):
         if self.config.html_logo:
             logobase = path.basename(self.config.html_logo)
             logotarget = path.join(self.outdir, '_static', logobase)
-            if not path.isfile(logobase):
-                self.warn('logo file %r does not exist' % logobase)
+            if not path.isfile(path.join(self.confdir, self.config.html_logo)):
+                self.warn('logo file %r does not exist' % self.config.html_logo)
             elif not path.isfile(logotarget):
                 copyfile(path.join(self.confdir, self.config.html_logo),
                          logotarget)
         if self.config.html_favicon:
             iconbase = path.basename(self.config.html_favicon)
             icontarget = path.join(self.outdir, '_static', iconbase)
-            if not path.isfile(icontarget):
+            if not path.isfile(path.join(self.confdir, self.config.html_favicon)):
+                self.warn('favicon file %r does not exist' % self.config.html_favicon)
+            elif not path.isfile(icontarget):
                 copyfile(path.join(self.confdir, self.config.html_favicon),
                          icontarget)
         self.info('done')
@@ -644,23 +660,25 @@ class StandaloneHTMLBuilder(Builder):
         their high res version.
         """
         Builder.post_process_images(self, doctree)
-        for node in doctree.traverse(nodes.image):
-            scale_keys = ('scale', 'width', 'height')
-            if not any((key in node) for key in scale_keys) or \
-               isinstance(node.parent, nodes.reference):
-                # docutils does unfortunately not preserve the
-                # ``target`` attribute on images, so we need to check
-                # the parent node here.
-                continue
-            uri = node['uri']
-            reference = nodes.reference('', '', internal=True)
-            if uri in self.images:
-                reference['refuri'] = posixpath.join(self.imgpath,
-                                                     self.images[uri])
-            else:
-                reference['refuri'] = uri
-            node.replace_self(reference)
-            reference.append(node)
+
+        if self.config.html_scaled_image_link:
+            for node in doctree.traverse(nodes.image):
+                scale_keys = ('scale', 'width', 'height')
+                if not any((key in node) for key in scale_keys) or \
+                   isinstance(node.parent, nodes.reference):
+                    # docutils does unfortunately not preserve the
+                    # ``target`` attribute on images, so we need to check
+                    # the parent node here.
+                    continue
+                uri = node['uri']
+                reference = nodes.reference('', '', internal=True)
+                if uri in self.images:
+                    reference['refuri'] = posixpath.join(self.imgpath,
+                                                         self.images[uri])
+                else:
+                    reference['refuri'] = uri
+                node.replace_self(reference)
+                reference.append(node)
 
     def load_indexer(self, docnames):
         keep = set(self.env.all_docs) - set(docnames)
@@ -702,7 +720,7 @@ class StandaloneHTMLBuilder(Builder):
         sidebars = None
         matched = None
         customsidebar = None
-        for pattern, patsidebars in self.config.html_sidebars.iteritems():
+        for pattern, patsidebars in iteritems(self.config.html_sidebars):
             if patmatch(pagename, pattern):
                 if matched:
                     if has_wildcard(pattern):
@@ -719,7 +737,7 @@ class StandaloneHTMLBuilder(Builder):
         if sidebars is None:
             # keep defaults
             pass
-        elif isinstance(sidebars, basestring):
+        elif isinstance(sidebars, string_types):
             # 0.x compatible mode: insert custom sidebar before searchbox
             customsidebar = sidebars
             sidebars = None
@@ -759,8 +777,10 @@ class StandaloneHTMLBuilder(Builder):
         self.add_sidebars(pagename, ctx)
         ctx.update(addctx)
 
-        self.app.emit('html-page-context', pagename, templatename,
-                      ctx, event_arg)
+        newtmpl = self.app.emit_firstresult('html-page-context', pagename,
+                                            templatename, ctx, event_arg)
+        if newtmpl:
+            templatename = newtmpl
 
         try:
             output = self.templates.render(templatename, ctx)
@@ -780,7 +800,7 @@ class StandaloneHTMLBuilder(Builder):
                 f.write(output)
             finally:
                 f.close()
-        except (IOError, OSError), err:
+        except (IOError, OSError) as err:
             self.warn("error writing file %s: %s" % (outfilename, err))
         if self.copysource and ctx.get('sourcename'):
             # copy the source file for the "show source" link
@@ -790,8 +810,9 @@ class StandaloneHTMLBuilder(Builder):
             copyfile(self.env.doc2path(pagename), source_name)
 
     def handle_finish(self):
-        self.dump_search_index()
-        self.dump_inventory()
+        if self.indexer:
+            self.finish_tasks.add_task(self.dump_search_index)
+        self.finish_tasks.add_task(self.dump_inventory)
 
     def dump_inventory(self):
         self.info(bold('dumping object inventory... '), nonl=True)
@@ -801,12 +822,11 @@ class StandaloneHTMLBuilder(Builder):
                      u'# Project: %s\n'
                      u'# Version: %s\n'
                      u'# The remainder of this file is compressed using zlib.\n'
-                     % (self.config.project, self.config.version)
-                    ).encode('utf-8'))
+                     % (self.config.project, self.config.version)).encode('utf-8'))
             compressor = zlib.compressobj(9)
-            for domainname, domain in self.env.domains.iteritems():
+            for domainname, domain in iteritems(self.env.domains):
                 for name, dispname, type, docname, anchor, prio in \
-                        domain.get_objects():
+                        sorted(domain.get_objects()):
                     if anchor.endswith(name):
                         # this can shorten the inventory by as much as 25%
                         anchor = anchor[:-len(name)] + '$'
@@ -815,15 +835,16 @@ class StandaloneHTMLBuilder(Builder):
                         dispname = u'-'
                     f.write(compressor.compress(
                         (u'%s %s:%s %s %s %s\n' % (name, domainname, type,
-                                                   prio, uri, dispname)
-                        ).encode('utf-8')))
+                                                   prio, uri, dispname)).encode('utf-8')))
             f.write(compressor.flush())
         finally:
             f.close()
         self.info('done')
 
     def dump_search_index(self):
-        self.info(bold('dumping search index... '), nonl=True)
+        self.info(
+            bold('dumping search index in %s ... ' % self.indexer.label()),
+            nonl=True)
         self.indexer.prune(self.env.all_docs)
         searchindexfn = path.join(self.outdir, self.searchindex_filename)
         # first write to a temporary file, so that if dumping fails,
@@ -852,13 +873,13 @@ class DirectoryHTMLBuilder(StandaloneHTMLBuilder):
         if docname == 'index':
             return ''
         if docname.endswith(SEP + 'index'):
-            return docname[:-5] # up to sep
+            return docname[:-5]  # up to sep
         return docname + SEP
 
     def get_outfilename(self, pagename):
         if pagename == 'index' or pagename.endswith(SEP + 'index'):
-            outfilename = path.join(self.outdir, os_path(pagename)
-                                    + self.out_suffix)
+            outfilename = path.join(self.outdir, os_path(pagename) +
+                                    self.out_suffix)
         else:
             outfilename = path.join(self.outdir, os_path(pagename),
                                     'index' + self.out_suffix)
@@ -885,7 +906,7 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
         if docname in self.env.all_docs:
             # all references are on the same page...
             return self.config.master_doc + self.out_suffix + \
-                   '#document-' + docname
+                '#document-' + docname
         else:
             # chances are this is a html_additional_page
             return docname + self.out_suffix
@@ -917,6 +938,23 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
         self.fix_refuris(tree)
         return tree
 
+    def assemble_toc_secnumbers(self):
+        # Assemble toc_secnumbers to resolve section numbers on SingleHTML.
+        # Merge all secnumbers to single secnumber.
+        #
+        # Note: current Sphinx has refid confliction in singlehtml mode.
+        #       To avoid the problem, it replaces key of secnumbers to
+        #       tuple of docname and refid.
+        #
+        #       There are related codes in inline_all_toctres() and
+        #       HTMLTranslter#add_secnumber().
+        new_secnumbers = {}
+        for docname, secnums in iteritems(self.env.toc_secnumbers):
+            for id, secnum in iteritems(secnums):
+                new_secnumbers[(docname, id)] = secnum
+
+        return {self.config.master_doc: new_secnumbers}
+
     def get_doc_context(self, docname, body, metatags):
         # no relation links...
         toc = self.env.get_toctree_for(self.config.master_doc, self, False)
@@ -952,6 +990,7 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
 
         self.info(bold('assembling single document... '), nonl=True)
         doctree = self.assemble_doctree()
+        self.env.toc_secnumbers = self.assemble_toc_secnumbers()
         self.info()
         self.info(bold('writing... '), nonl=True)
         self.write_doc_serialized(self.config.master_doc, doctree)
@@ -1003,6 +1042,7 @@ class SerializingHTMLBuilder(StandaloneHTMLBuilder):
     def init(self):
         self.config_hash = ''
         self.tags_hash = ''
+        self.imagedir = '_images'
         self.theme = None       # no theme necessary
         self.templates = None   # no template bridge necessary
         self.init_translator_class()
@@ -1012,7 +1052,7 @@ class SerializingHTMLBuilder(StandaloneHTMLBuilder):
         if docname == 'index':
             return ''
         if docname.endswith(SEP + 'index'):
-            return docname[:-5] # up to sep
+            return docname[:-5]  # up to sep
         return docname + SEP
 
     def dump_context(self, context, filename):
@@ -1034,8 +1074,9 @@ class SerializingHTMLBuilder(StandaloneHTMLBuilder):
             outfilename = path.join(self.outdir,
                                     os_path(pagename) + self.out_suffix)
 
-        self.app.emit('html-page-context', pagename, templatename,
-                      ctx, event_arg)
+        # we're not taking the return value here, since no template is
+        # actually rendered
+        self.app.emit('html-page-context', pagename, templatename, ctx, event_arg)
 
         ensuredir(path.dirname(outfilename))
         self.dump_context(ctx, outfilename)
@@ -1098,8 +1139,4 @@ class JSONHTMLBuilder(SerializingHTMLBuilder):
     searchindex_filename = 'searchindex.json'
 
     def init(self):
-        if jsonimpl.json is None:
-            raise SphinxError(
-                'The module simplejson (or json in Python >= 2.6) '
-                'is not available. The JSONHTMLBuilder builder will not work.')
         SerializingHTMLBuilder.init(self)
