@@ -5,25 +5,29 @@
 
     Build configuration file handling.
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import os
 import re
-import sys
-from os import path
+from os import path, environ
+import shlex
+
+from six import PY3, iteritems, string_types, binary_type, integer_types
 
 from sphinx.errors import ConfigError
 from sphinx.locale import l_
-from sphinx.util.osutil import make_filename
-from sphinx.util.pycompat import bytes, b, execfile_
+from sphinx.util.osutil import make_filename, cd
+from sphinx.util.pycompat import execfile_
 
-nonascii_re = re.compile(b(r'[\x80-\xff]'))
+nonascii_re = re.compile(br'[\x80-\xff]')
 
 CONFIG_SYNTAX_ERROR = "There is a syntax error in your configuration file: %s"
-if sys.version_info >= (3, 0):
+if PY3:
     CONFIG_SYNTAX_ERROR += "\nDid you change the syntax from 2.x to 3.x?"
+CONFIG_EXIT_ERROR = "The configuration file (or one of the modules it imports) " \
+                    "called sys.exit()"
+
 
 class Config(object):
     """
@@ -48,13 +52,10 @@ class Config(object):
         locale_dirs = ([], 'env'),
 
         master_doc = ('contents', 'env'),
-        source_suffix = ('.rst', 'env'),
+        source_suffix = (['.rst'], 'env'),
         source_encoding = ('utf-8-sig', 'env'),
+        source_parsers = ({}, 'env'),
         exclude_patterns = ([], 'env'),
-        # the next three are all deprecated now
-        unused_docs = ([], 'env'),
-        exclude_trees = ([], 'env'),
-        exclude_dirnames = ([], 'env'),
         default_role = (None, 'env'),
         add_function_parentheses = (True, 'env'),
         add_module_names = (True, 'env'),
@@ -62,6 +63,7 @@ class Config(object):
         show_authors = (False, 'env'),
         pygments_style = (None, 'html'),
         highlight_language = ('python', 'env'),
+        highlight_options = ({}, 'env'),
         templates_path = ([], 'html'),
         template_bridge = (None, 'html'),
         keep_warnings = (False, 'env'),
@@ -71,15 +73,22 @@ class Config(object):
         trim_doctest_flags = (True, 'env'),
         primary_domain = ('py', 'env'),
         needs_sphinx = (None, None),
+        needs_extensions = ({}, None),
         nitpicky = (False, 'env'),
         nitpick_ignore = ([], 'html'),
+        numfig = (False, 'env'),
+        numfig_secnum_depth = (1, 'env'),
+        numfig_format = ({'figure': l_('Fig. %s'),
+                          'table': l_('Table %s'),
+                          'code-block': l_('Listing %s')},
+                         'env'),
 
         # HTML options
-        html_theme = ('default', 'html'),
+        html_theme = ('alabaster', 'html'),
         html_theme_path = ([], 'html'),
         html_theme_options = ({}, 'html'),
         html_title = (lambda self: l_('%s %s documentation') %
-                                   (self.project, self.release),
+                      (self.project, self.release),
                       'html'),
         html_short_title = (lambda self: self.html_title, 'html'),
         html_style = (None, 'html'),
@@ -112,6 +121,7 @@ class Config(object):
         html_search_language = (None, 'html'),
         html_search_options = ({}, 'html'),
         html_search_scorer = ('', None),
+        html_scaled_image_link = (True, 'html'),
 
         # HTML help only options
         htmlhelp_basename = (lambda self: make_filename(self.project), None),
@@ -122,6 +132,35 @@ class Config(object):
         # Devhelp only options
         devhelp_basename = (lambda self: make_filename(self.project), None),
 
+        # Apple help options
+        applehelp_bundle_name = (lambda self: make_filename(self.project),
+                                 'applehelp'),
+        applehelp_bundle_id = (None, 'applehelp'),
+        applehelp_dev_region = ('en-us', 'applehelp'),
+        applehelp_bundle_version = ('1', 'applehelp'),
+        applehelp_icon = (None, 'applehelp'),
+        applehelp_kb_product = (lambda self: '%s-%s' %
+                                (make_filename(self.project), self.release),
+                                'applehelp'),
+        applehelp_kb_url = (None, 'applehelp'),
+        applehelp_remote_url = (None, 'applehelp'),
+        applehelp_index_anchors = (False, 'applehelp'),
+        applehelp_min_term_length = (None, 'applehelp'),
+        applehelp_stopwords = (lambda self: self.language or 'en', 'applehelp'),
+        applehelp_locale = (lambda self: self.language or 'en', 'applehelp'),
+        applehelp_title = (lambda self: self.project + ' Help', 'applehelp'),
+        applehelp_codesign_identity = (lambda self:
+                                       environ.get('CODE_SIGN_IDENTITY', None),
+                                       'applehelp'),
+        applehelp_codesign_flags = (lambda self:
+                                    shlex.split(
+                                        environ.get('OTHER_CODE_SIGN_FLAGS',
+                                                    '')),
+                                    'applehelp'),
+        applehelp_indexer_path = ('/usr/bin/hiutil', 'applehelp'),
+        applehelp_codesign_path = ('/usr/bin/codesign', 'applehelp'),
+        applehelp_disable_external_tools = (False, None),
+
         # Epub options
         epub_basename = (lambda self: make_filename(self.project), None),
         epub_theme = ('epub', 'html'),
@@ -205,6 +244,10 @@ class Config(object):
 
         # gettext options
         gettext_compact = (True, 'gettext'),
+        gettext_location = (True, 'gettext'),
+        gettext_uuid = (False, 'gettext'),
+        gettext_auto_build = (True, 'env'),
+        gettext_additional_targets = ([], 'env'),
 
         # XML options
         xml_pretty = (True, 'env'),
@@ -214,23 +257,24 @@ class Config(object):
         self.overrides = overrides
         self.values = Config.config_values.copy()
         config = {}
-        if "extensions" in overrides:
-            config["extensions"] = overrides["extensions"]
+        if 'extensions' in overrides:  # XXX do we need this?
+            if isinstance(overrides['extensions'], string_types):
+                config['extensions'] = overrides.pop('extensions').split(',')
+            else:
+                config['extensions'] = overrides.pop('extensions')
         if dirname is not None:
             config_file = path.join(dirname, filename)
             config['__file__'] = config_file
             config['tags'] = tags
-            olddir = os.getcwd()
-            try:
+            with cd(dirname):
                 # we promise to have the config dir as current dir while the
                 # config file is executed
-                os.chdir(dirname)
                 try:
                     execfile_(filename, config)
-                except SyntaxError, err:
+                except SyntaxError as err:
                     raise ConfigError(CONFIG_SYNTAX_ERROR % err)
-            finally:
-                os.chdir(olddir)
+                except SystemExit:
+                    raise ConfigError(CONFIG_EXIT_ERROR)
 
         self._raw_config = config
         # these two must be preinitialized because extensions can add their
@@ -238,28 +282,78 @@ class Config(object):
         self.setup = config.get('setup', None)
         self.extensions = config.get('extensions', [])
 
+    def check_types(self, warn):
+        # check all values for deviation from the default value's type, since
+        # that can result in TypeErrors all over the place
+        # NB. since config values might use l_() we have to wait with calling
+        # this method until i18n is initialized
+        for name in self._raw_config:
+            if name not in Config.config_values:
+                continue  # we don't know a default value
+            default, dummy_rebuild = Config.config_values[name]
+            if hasattr(default, '__call__'):
+                default = default(self)  # could invoke l_()
+            if default is None:
+                continue
+            current = self[name]
+            if type(current) is type(default):
+                continue
+            common_bases = (set(type(current).__bases__ + (type(current),)) &
+                            set(type(default).__bases__))
+            common_bases.discard(object)
+            if common_bases:
+                continue  # at least we share a non-trivial base class
+            warn("the config value %r has type `%s', defaults to `%s.'" %
+                 (name, type(current).__name__, type(default).__name__))
+
     def check_unicode(self, warn):
         # check all string values for non-ASCII characters in bytestrings,
         # since that can result in UnicodeErrors all over the place
-        for name, value in self._raw_config.iteritems():
-            if isinstance(value, bytes) and nonascii_re.search(value):
+        for name, value in iteritems(self._raw_config):
+            if isinstance(value, binary_type) and nonascii_re.search(value):
                 warn('the config value %r is set to a string with non-ASCII '
                      'characters; this can lead to Unicode errors occurring. '
-                     'Please use Unicode strings, e.g. %r.' % (name, u'Content')
-                )
+                     'Please use Unicode strings, e.g. %r.' % (name, u'Content'))
 
-    def init_values(self):
+    def init_values(self, warn):
         config = self._raw_config
-        for valname, value in self.overrides.iteritems():
+        for valname, value in iteritems(self.overrides):
             if '.' in valname:
                 realvalname, key = valname.split('.', 1)
                 config.setdefault(realvalname, {})[key] = value
+                continue
+            elif valname not in self.values:
+                warn('unknown config value %r in override, ignoring' % valname)
+                continue
+            defvalue = self.values[valname][0]
+            if isinstance(value, string_types):
+                if isinstance(defvalue, dict):
+                    warn('cannot override dictionary config setting %r, '
+                         'ignoring (use %r to set individual elements)' %
+                         (valname, valname + '.key=value'))
+                    continue
+                elif isinstance(defvalue, list):
+                    config[valname] = value.split(',')
+                elif isinstance(defvalue, integer_types):
+                    try:
+                        config[valname] = int(value)
+                    except ValueError:
+                        warn('invalid number %r for config value %r, ignoring'
+                             % (value, valname))
+                elif hasattr(defvalue, '__call__'):
+                    config[valname] = value
+                elif defvalue is not None and not isinstance(defvalue, string_types):
+                    warn('cannot override config setting %r with unsupported '
+                         'type, ignoring' % valname)
+                else:
+                    config[valname] = value
             else:
                 config[valname] = value
         for name in config:
             if name in self.values:
                 self.__dict__[name] = config[name]
-        del self._raw_config
+        if isinstance(self.source_suffix, string_types):
+            self.source_suffix = [self.source_suffix]
 
     def __getattr__(self, name):
         if name.startswith('_'):
