@@ -6,16 +6,19 @@
     Check Python modules and C API for coverage.  Mostly written by Josip
     Dzolonga for the Google Highly Open Participation contest.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
 import glob
 import inspect
-import cPickle as pickle
 from os import path
 
+from six import iteritems
+from six.moves import cPickle as pickle
+
+import sphinx
 from sphinx.builders import Builder
 
 
@@ -24,6 +27,7 @@ def write_header(f, text, char='-'):
     f.write(text + '\n')
     f.write(char * len(text) + '\n')
 
+
 def compile_regex_list(name, exps, warnfunc):
     lst = []
     for exp in exps:
@@ -52,7 +56,7 @@ class CoverageBuilder(Builder):
                 self.warn('invalid regex %r in coverage_c_regexes' % exp)
 
         self.c_ignorexps = {}
-        for (name, exps) in self.config.coverage_ignore_c_items.iteritems():
+        for (name, exps) in iteritems(self.config.coverage_ignore_c_items):
             self.c_ignorexps[name] = compile_regex_list(
                 'coverage_ignore_c_items', exps, self.warn)
         self.mod_ignorexps = compile_regex_list(
@@ -81,7 +85,7 @@ class CoverageBuilder(Builder):
         # Fetch all the info from the header files
         c_objects = self.env.domaindata['c']['objects']
         for filename in self.c_sourcefiles:
-            undoc = []
+            undoc = set()
             f = open(filename, 'r')
             try:
                 for line in f:
@@ -94,7 +98,7 @@ class CoverageBuilder(Builder):
                                     if exp.match(name):
                                         break
                                 else:
-                                    undoc.append((key, name))
+                                    undoc.add((key, name))
                             continue
             finally:
                 f.close()
@@ -105,12 +109,13 @@ class CoverageBuilder(Builder):
         output_file = path.join(self.outdir, 'c.txt')
         op = open(output_file, 'w')
         try:
-            write_header(op, 'Undocumented C API elements', '=')
+            if self.config.coverage_write_headline:
+                write_header(op, 'Undocumented C API elements', '=')
             op.write('\n')
 
-            for filename, undoc in self.c_undoc.iteritems():
+            for filename, undoc in iteritems(self.c_undoc):
                 write_header(op, filename)
-                for typ, name in undoc:
+                for typ, name in sorted(undoc):
                     op.write(' * %-50s [%9s]\n' % (name, typ))
                 op.write('\n')
         finally:
@@ -120,6 +125,8 @@ class CoverageBuilder(Builder):
         objects = self.env.domaindata['py']['objects']
         modules = self.env.domaindata['py']['modules']
 
+        skip_undoc = self.config.coverage_skip_undoc_in_source
+
         for mod_name in modules:
             ignore = False
             for exp in self.mod_ignorexps:
@@ -131,7 +138,7 @@ class CoverageBuilder(Builder):
 
             try:
                 mod = __import__(mod_name, fromlist=['foo'])
-            except ImportError, err:
+            except ImportError as err:
                 self.warn('module %s could not be imported: %s' %
                           (mod_name, err))
                 self.py_undoc[mod_name] = {'error': err}
@@ -160,6 +167,8 @@ class CoverageBuilder(Builder):
                             if exp.match(name):
                                 break
                         else:
+                            if skip_undoc and not obj.__doc__:
+                                continue
                             funcs.append(name)
                 elif inspect.isclass(obj):
                     for exp in self.cls_ignorexps:
@@ -167,17 +176,27 @@ class CoverageBuilder(Builder):
                             break
                     else:
                         if full_name not in objects:
+                            if skip_undoc and not obj.__doc__:
+                                continue
                             # not documented at all
                             classes[name] = []
                             continue
 
                         attrs = []
 
-                        for attr_name, attr in inspect.getmembers(
-                               obj, inspect.ismethod):
+                        for attr_name in dir(obj):
+                            if attr_name not in obj.__dict__:
+                                continue
+                            attr = getattr(obj, attr_name)
+                            if not (inspect.ismethod(attr) or
+                                    inspect.isfunction(attr)):
+                                continue
                             if attr_name[0] == '_':
                                 # starts with an underscore, ignore it
                                 continue
+                            if skip_undoc and not attr.__doc__:
+                                # skip methods without docstring if wished
+                                continue
 
                             full_attr_name = '%s.%s' % (full_name, attr_name)
                             if full_attr_name not in objects:
@@ -194,10 +213,9 @@ class CoverageBuilder(Builder):
         op = open(output_file, 'w')
         failed = []
         try:
-            write_header(op, 'Undocumented Python objects', '=')
-
-            keys = self.py_undoc.keys()
-            keys.sort()
+            if self.config.coverage_write_headline:
+                write_header(op, 'Undocumented Python objects', '=')
+            keys = sorted(self.py_undoc.keys())
             for name in keys:
                 undoc = self.py_undoc[name]
                 if 'error' in undoc:
@@ -213,11 +231,12 @@ class CoverageBuilder(Builder):
                         op.write('\n')
                     if undoc['classes']:
                         op.write('Classes:\n')
-                        for name, methods in undoc['classes'].iteritems():
+                        for name, methods in sorted(
+                                iteritems(undoc['classes'])):
                             if not methods:
                                 op.write(' * %s\n' % name)
                             else:
-                                op.write(' * %s -- missing methods:\n' % name)
+                                op.write(' * %s -- missing methods:\n\n' % name)
                                 op.writelines('   - %s\n' % x for x in methods)
                         op.write('\n')
 
@@ -245,3 +264,6 @@ def setup(app):
     app.add_config_value('coverage_c_path', [], False)
     app.add_config_value('coverage_c_regexes', {}, False)
     app.add_config_value('coverage_ignore_c_items', {}, False)
+    app.add_config_value('coverage_write_headline', True, False)
+    app.add_config_value('coverage_skip_undoc_in_source', False, False)
+    return {'version': sphinx.__version__, 'parallel_read_safe': True}
