@@ -5,7 +5,7 @@
 
     Render math in HTML via dvipng.
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -14,21 +14,21 @@ import codecs
 import shutil
 import tempfile
 import posixpath
-from os import path, getcwd, chdir
+from os import path
 from subprocess import Popen, PIPE
-try:
-    from hashlib import sha1 as sha
-except ImportError:
-    from sha import sha
+from hashlib import sha1
 
+from six import text_type
 from docutils import nodes
 
+import sphinx
 from sphinx.errors import SphinxError
 from sphinx.util.png import read_png_depth, write_png_depth
-from sphinx.util.osutil import ensuredir, ENOENT
-from sphinx.util.pycompat import b, sys_encoding
+from sphinx.util.osutil import ensuredir, ENOENT, cd
+from sphinx.util.pycompat import sys_encoding
 from sphinx.ext.mathbase import setup_math as mathbase_setup, wrap_displaymath
 
+
 class MathExtError(SphinxError):
     category = 'Math extension error'
 
@@ -66,7 +66,8 @@ DOC_BODY_PREVIEW = r'''
 \end{document}
 '''
 
-depth_re = re.compile(b(r'\[\d+ depth=(-?\d+)\]'))
+depth_re = re.compile(br'\[\d+ depth=(-?\d+)\]')
+
 
 def render_math(self, math):
     """Render the LaTeX math expression *math* using latex and dvipng.
@@ -85,9 +86,9 @@ def render_math(self, math):
     latex = DOC_HEAD + self.builder.config.pngmath_latex_preamble
     latex += (use_preview and DOC_BODY_PREVIEW or DOC_BODY) % math
 
-    shasum = "%s.png" % sha(latex.encode('utf-8')).hexdigest()
+    shasum = "%s.png" % sha1(latex.encode('utf-8')).hexdigest()
     relfn = posixpath.join(self.builder.imgpath, 'math', shasum)
-    outfn = path.join(self.builder.outdir, '_images', 'math', shasum)
+    outfn = path.join(self.builder.outdir, self.builder.imagedir, 'math', shasum)
     if path.isfile(outfn):
         depth = read_png_depth(outfn)
         return relfn, depth
@@ -117,13 +118,10 @@ def render_math(self, math):
     ltx_args.extend(self.builder.config.pngmath_latex_args)
     ltx_args.append('math.tex')
 
-    curdir = getcwd()
-    chdir(tempdir)
-
-    try:
+    with cd(tempdir):
         try:
             p = Popen(ltx_args, stdout=PIPE, stderr=PIPE)
-        except OSError, err:
+        except OSError as err:
             if err.errno != ENOENT:   # No such file or directory
                 raise
             self.builder.warn('LaTeX command %r cannot be run (needed for math '
@@ -131,8 +129,6 @@ def render_math(self, math):
                               self.builder.config.pngmath_latex)
             self.builder._mathpng_warned_latex = True
             return None, None
-    finally:
-        chdir(curdir)
 
     stdout, stderr = p.communicate()
     if p.returncode != 0:
@@ -150,7 +146,7 @@ def render_math(self, math):
     dvipng_args.append(path.join(tempdir, 'math.dvi'))
     try:
         p = Popen(dvipng_args, stdout=PIPE, stderr=PIPE)
-    except OSError, err:
+    except OSError as err:
         if err.errno != ENOENT:   # No such file or directory
             raise
         self.builder.warn('dvipng command %r cannot be run (needed for math '
@@ -172,6 +168,7 @@ def render_math(self, math):
 
     return relfn, depth
 
+
 def cleanup_tempdir(app, exc):
     if exc:
         return
@@ -182,16 +179,18 @@ def cleanup_tempdir(app, exc):
     except Exception:
         pass
 
+
 def get_tooltip(self, node):
     if self.builder.config.pngmath_add_tooltips:
         return ' alt="%s"' % self.encode(node['latex']).strip()
     return ''
 
+
 def html_visit_math(self, node):
     try:
         fname, depth = render_math(self, '$'+node['latex']+'$')
-    except MathExtError, exc:
-        msg = unicode(exc)
+    except MathExtError as exc:
+        msg = text_type(exc)
         sm = nodes.system_message(msg, type='WARNING', level=2,
                                   backrefs=[], source=node['latex'])
         sm.walkabout(self)
@@ -202,12 +201,13 @@ def html_visit_math(self, node):
         self.body.append('<span class="math">%s</span>' %
                          self.encode(node['latex']).strip())
     else:
-        c  = ('<img class="math" src="%s"' % fname) + get_tooltip(self, node)
+        c = ('<img class="math" src="%s"' % fname) + get_tooltip(self, node)
         if depth is not None:
             c += ' style="vertical-align: %dpx"' % (-depth)
         self.body.append(c + '/>')
     raise nodes.SkipNode
 
+
 def html_visit_displaymath(self, node):
     if node['nowrap']:
         latex = node['latex']
@@ -215,7 +215,7 @@ def html_visit_displaymath(self, node):
         latex = wrap_displaymath(node['latex'], None)
     try:
         fname, depth = render_math(self, latex)
-    except MathExtError, exc:
+    except MathExtError as exc:
         sm = nodes.system_message(str(exc), type='WARNING', level=2,
                                   backrefs=[], source=node['latex'])
         sm.walkabout(self)
@@ -230,8 +230,8 @@ def html_visit_displaymath(self, node):
         self.body.append('<span class="math">%s</span></p>\n</div>' %
                          self.encode(node['latex']).strip())
     else:
-        self.body.append(('<img src="%s"' % fname) + get_tooltip(self, node)
-                         + '/></p>\n</div>')
+        self.body.append(('<img src="%s"' % fname) + get_tooltip(self, node) +
+                         '/></p>\n</div>')
     raise nodes.SkipNode
 
 
@@ -247,3 +247,4 @@ def setup(app):
     app.add_config_value('pngmath_latex_preamble', '', 'html')
     app.add_config_value('pngmath_add_tooltips', True, 'html')
     app.connect('build-finished', cleanup_tempdir)
+    return {'version': sphinx.__version__, 'parallel_read_safe': True}
