@@ -8,17 +8,25 @@
     all todos of your project and lists them along with a backlink to the
     original location.
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from docutils import nodes
 
+import sphinx
+from sphinx.locale import _
 from sphinx.environment import NoUri
+from sphinx.util.nodes import set_source_info
 from sphinx.util.compat import Directive, make_admonition
 
-class todo_node(nodes.Admonition, nodes.Element): pass
-class todolist(nodes.General, nodes.Element): pass
+
+class todo_node(nodes.Admonition, nodes.Element):
+    pass
+
+
+class todolist(nodes.General, nodes.Element):
+    pass
 
 
 class Todo(Directive):
@@ -34,28 +42,40 @@ class Todo(Directive):
 
     def run(self):
         env = self.state.document.settings.env
-
-        targetid = "todo-%s" % env.index_num
-        env.index_num += 1
+        targetid = 'index-%s' % env.new_serialno('index')
         targetnode = nodes.target('', '', ids=[targetid])
 
         ad = make_admonition(todo_node, self.name, [_('Todo')], self.options,
                              self.content, self.lineno, self.content_offset,
                              self.block_text, self.state, self.state_machine)
+        set_source_info(self, ad[0])
+        return [targetnode] + ad
 
-        # Attach a list of all todos to the environment,
-        # the todolist works with the collected todo nodes
-        if not hasattr(env, 'todo_all_todos'):
-            env.todo_all_todos = []
+
+def process_todos(app, doctree):
+    # collect all todos in the environment
+    # this is not done in the directive itself because it some transformations
+    # must have already been run, e.g. substitutions
+    env = app.builder.env
+    if not hasattr(env, 'todo_all_todos'):
+        env.todo_all_todos = []
+    for node in doctree.traverse(todo_node):
+        try:
+            targetnode = node.parent[node.parent.index(node) - 1]
+            if not isinstance(targetnode, nodes.target):
+                raise IndexError
+        except IndexError:
+            targetnode = None
+        newnode = node.deepcopy()
+        del newnode['ids']
         env.todo_all_todos.append({
             'docname': env.docname,
-            'lineno': self.lineno,
-            'todo': ad[0].deepcopy(),
+            'source': node.source or env.doc2path(env.docname),
+            'lineno': node.line,
+            'todo': newnode,
             'target': targetnode,
         })
 
-        return [targetnode] + ad
-
 
 class TodoList(Directive):
     """
@@ -94,17 +114,17 @@ def process_todo_nodes(app, doctree, fromdocname):
         content = []
 
         for todo_info in env.todo_all_todos:
-            para = nodes.paragraph()
-            filename = env.doc2path(todo_info['docname'], base=None)
-            description = (
-                _('(The original entry is located in %s, line %d and '
-                  'can be found ') % (filename, todo_info['lineno']))
-            para += nodes.Text(description, description)
+            para = nodes.paragraph(classes=['todo-source'])
+            description = _('(The <<original entry>> is located in '
+                            ' %s, line %d.)') % \
+                (todo_info['source'], todo_info['lineno'])
+            desc1 = description[:description.find('<<')]
+            desc2 = description[description.find('>>')+2:]
+            para += nodes.Text(desc1, desc1)
 
             # Create a reference
-            newnode = nodes.reference('', '')
-            innernode = nodes.emphasis(_('here'), _('here'))
-            newnode['refdocname'] = todo_info['docname']
+            newnode = nodes.reference('', '', internal=True)
+            innernode = nodes.emphasis(_('original entry'), _('original entry'))
             try:
                 newnode['refuri'] = app.builder.get_relative_uri(
                     fromdocname, todo_info['docname'])
@@ -114,7 +134,7 @@ def process_todo_nodes(app, doctree, fromdocname):
                 pass
             newnode.append(innernode)
             para += newnode
-            para += nodes.Text('.)', '.)')
+            para += nodes.Text(desc2, desc2)
 
             # (Recursively) resolve references in the todo content
             todo_entry = todo_info['todo']
@@ -135,23 +155,37 @@ def purge_todos(app, env, docname):
                           if todo['docname'] != docname]
 
 
+def merge_info(app, env, docnames, other):
+    if not hasattr(other, 'todo_all_todos'):
+        return
+    if not hasattr(env, 'todo_all_todos'):
+        env.todo_all_todos = []
+    env.todo_all_todos.extend(other.todo_all_todos)
+
+
 def visit_todo_node(self, node):
     self.visit_admonition(node)
 
+
 def depart_todo_node(self, node):
     self.depart_admonition(node)
 
+
 def setup(app):
-    app.add_config_value('todo_include_todos', False, False)
+    app.add_config_value('todo_include_todos', False, 'html')
 
     app.add_node(todolist)
     app.add_node(todo_node,
                  html=(visit_todo_node, depart_todo_node),
                  latex=(visit_todo_node, depart_todo_node),
-                 text=(visit_todo_node, depart_todo_node))
+                 text=(visit_todo_node, depart_todo_node),
+                 man=(visit_todo_node, depart_todo_node),
+                 texinfo=(visit_todo_node, depart_todo_node))
 
     app.add_directive('todo', Todo)
     app.add_directive('todolist', TodoList)
+    app.connect('doctree-read', process_todos)
     app.connect('doctree-resolved', process_todo_nodes)
     app.connect('env-purge-doc', purge_todos)
-
+    app.connect('env-merge-info', merge_info)
+    return {'version': sphinx.__version__, 'parallel_read_safe': True}
