@@ -5,7 +5,7 @@
 
     Docutils transforms used by Sphinx when reading documents.
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -20,10 +20,11 @@ from docutils.transforms.parts import ContentsFilter
 from sphinx import addnodes
 from sphinx.locale import _, init as init_locale
 from sphinx.util import split_index_msg
-from sphinx.util.nodes import traverse_translatable_index, extract_messages
+from sphinx.util.nodes import (
+    traverse_translatable_index, extract_messages, LITERAL_TYPE_NODES, IMAGE_TYPE_NODES,
+)
 from sphinx.util.osutil import ustrftime, find_catalog
-from sphinx.util.compat import docutils_version
-from sphinx.util.pycompat import all
+from sphinx.util.pycompat import indent
 from sphinx.domains.std import (
     make_term_from_paragraph_node,
     make_termnodes_from_paragraph_node,
@@ -36,6 +37,7 @@ default_substitutions = set([
     'today',
 ])
 
+
 class DefaultSubstitutions(Transform):
     """
     Replace some substitutions if they aren't defined in the document.
@@ -70,10 +72,10 @@ class MoveModuleTargets(Transform):
         for node in self.document.traverse(nodes.target):
             if not node['ids']:
                 continue
-            if (node.has_key('ismod') and
-                node.parent.__class__ is nodes.section and
-                # index 0 is the section title node
-                node.parent.index(node) == 1):
+            if ('ismod' in node and
+                    node.parent.__class__ is nodes.section and
+                    # index 0 is the section title node
+                    node.parent.index(node) == 1):
                 node.parent['ids'][0:0] = node['ids']
                 node.parent.remove(node)
 
@@ -88,10 +90,10 @@ class HandleCodeBlocks(Transform):
         # move doctest blocks out of blockquotes
         for node in self.document.traverse(nodes.block_quote):
             if all(isinstance(child, nodes.doctest_block) for child
-                     in node.children):
+                   in node.children):
                 node.replace_self(node.children)
         # combine successive doctest blocks
-        #for node in self.document.traverse(nodes.doctest_block):
+        # for node in self.document.traverse(nodes.doctest_block):
         #    if node not in node.parent.children:
         #        continue
         #    parindex = node.parent.index(node)
@@ -102,6 +104,31 @@ class HandleCodeBlocks(Transform):
         #        del node.parent[parindex+1]
 
 
+class AutoNumbering(Transform):
+    """
+    Register IDs of tables, figures and literal_blocks to assign numbers.
+    """
+    default_priority = 210
+
+    def apply(self):
+        def has_child(node, cls):
+            return any(isinstance(child, cls) for child in node)
+
+        for node in self.document.traverse(nodes.Element):
+            if isinstance(node, nodes.figure):
+                if has_child(node, nodes.caption):
+                    self.document.note_implicit_target(node)
+            elif isinstance(node, nodes.image):
+                if has_child(node.parent, nodes.caption):
+                    self.document.note_implicit_target(node.parent)
+            elif isinstance(node, nodes.table):
+                if has_child(node, nodes.title):
+                    self.document.note_implicit_target(node)
+            elif isinstance(node, nodes.literal_block):
+                if has_child(node.parent, nodes.caption):
+                    self.document.note_implicit_target(node.parent)
+
+
 class SortIds(Transform):
     """
     Sort secion IDs so that the "id[0-9]+" one comes last.
@@ -132,6 +159,34 @@ class CitationReferences(Transform):
             citnode.parent.replace(citnode, refnode)
 
 
+TRANSLATABLE_NODES = {
+    'literal-block': nodes.literal_block,
+    'doctest-block': nodes.doctest_block,
+    'raw': nodes.raw,
+    'index': addnodes.index,
+    'image': nodes.image,
+}
+
+
+class ExtraTranslatableNodes(Transform):
+    """
+    make nodes translatable
+    """
+    default_priority = 10
+
+    def apply(self):
+        targets = self.document.settings.env.config.gettext_additional_targets
+        target_nodes = [v for k, v in TRANSLATABLE_NODES.items() if k in targets]
+        if not target_nodes:
+            return
+
+        def is_translatable_node(node):
+            return isinstance(node, tuple(target_nodes))
+
+        for node in self.document.traverse(is_translatable_node):
+            node['translatable'] = True
+
+
 class CustomLocaleReporter(object):
     """
     Replacer for document.reporter.get_source_and_line method.
@@ -143,10 +198,7 @@ class CustomLocaleReporter(object):
         self.source, self.line = source, line
 
     def set_reporter(self, document):
-        if docutils_version < (0, 9):
-            document.reporter.locator = self.get_source_and_line
-        else:
-            document.reporter.get_source_and_line = self.get_source_and_line
+        document.reporter.get_source_and_line = self.get_source_and_line
 
     def get_source_and_line(self, lineno=None):
         return self.source, self.line
@@ -156,14 +208,15 @@ class Locale(Transform):
     """
     Replace translatable nodes with their translated doctree.
     """
-    default_priority = 0
+    default_priority = 20
 
     def apply(self):
         env = self.document.settings.env
         settings, source = self.document.settings, self.document['source']
         # XXX check if this is reliable
         assert source.startswith(env.srcdir)
-        docname = path.splitext(relative_path(env.srcdir, source))[0]
+        docname = path.splitext(relative_path(path.join(env.srcdir, 'dummy'),
+                                              source))[0]
         textdomain = find_catalog(docname,
                                   self.document.settings.gettext_compact)
 
@@ -177,7 +230,7 @@ class Locale(Transform):
 
         parser = RSTParser()
 
-        #phase1: replace reference ids with translated names
+        # phase1: replace reference ids with translated names
         for node, msg in extract_messages(self.document):
             msgstr = catalog.gettext(msg)
             # XXX add marker to untranslated parts
@@ -193,6 +246,11 @@ class Locale(Transform):
                 msgstr += '\n\n   dummy literal'
                 # dummy literal node will discard by 'patch = patch[0]'
 
+            # literalblock need literal block notation to avoid it become
+            # paragraph.
+            if isinstance(node, LITERAL_TYPE_NODES):
+                msgstr = '::\n\n' + indent(msgstr, ' '*3)
+
             patch = new_document(source, settings)
             CustomLocaleReporter(node.source, node.line).set_reporter(patch)
             parser.parse(msgstr, patch)
@@ -202,7 +260,7 @@ class Locale(Transform):
                 pass
             # XXX doctest and other block markup
             if not isinstance(patch, nodes.paragraph):
-                continue # skip for now
+                continue  # skip for now
 
             processed = False  # skip flag
 
@@ -226,23 +284,35 @@ class Locale(Transform):
                     # * if explicit: _id is label. title node need another id.
                     # * if not explicit:
                     #
-                    #   * _id is None:
+                    #   * if _id is None:
                     #
-                    #     _id is None means _id was duplicated.
-                    #     old_name entry still exists in nameids and
-                    #     nametypes for another duplicated entry.
+                    #     _id is None means:
                     #
-                    #   * _id is provided: bellow process
-                    if not explicit and _id:
-                        # _id was not duplicated.
-                        # remove old_name entry from document ids database
-                        # to reuse original _id.
-                        self.document.nameids.pop(old_name, None)
-                        self.document.nametypes.pop(old_name, None)
-                        self.document.ids.pop(_id, None)
+                    #     1. _id was not provided yet.
+                    #
+                    #     2. _id was duplicated.
+                    #
+                    #        old_name entry still exists in nameids and
+                    #        nametypes for another duplicated entry.
+                    #
+                    #   * if _id is provided: bellow process
+                    if _id:
+                        if not explicit:
+                            # _id was not duplicated.
+                            # remove old_name entry from document ids database
+                            # to reuse original _id.
+                            self.document.nameids.pop(old_name, None)
+                            self.document.nametypes.pop(old_name, None)
+                            self.document.ids.pop(_id, None)
 
-                    # re-entry with new named section node.
-                    self.document.note_implicit_target(section_node)
+                        # re-entry with new named section node.
+                        #
+                        # Note: msgnode that is a second parameter of the
+                        # `note_implicit_target` is not necessary here because
+                        # section_node has been noted previously on rst parsing by
+                        # `docutils.parsers.rst.states.RSTState.new_subsection()`
+                        # and already has `system_message` if needed.
+                        self.document.note_implicit_target(section_node)
 
                     # replace target's refname to new target name
                     def is_named_target(node):
@@ -279,15 +349,14 @@ class Locale(Transform):
                 node.children = patch.children
                 node['translated'] = True
 
-
-        #phase2: translation
+        # phase2: translation
         for node, msg in extract_messages(self.document):
             if node.get('translated', False):
                 continue
 
             msgstr = catalog.gettext(msg)
             # XXX add marker to untranslated parts
-            if not msgstr or msgstr == msg: # as-of-yet untranslated
+            if not msgstr or msgstr == msg:  # as-of-yet untranslated
                 continue
 
             # Avoid "Literal block expected; none found." warnings.
@@ -298,6 +367,11 @@ class Locale(Transform):
                 msgstr += '\n\n   dummy literal'
                 # dummy literal node will discard by 'patch = patch[0]'
 
+            # literalblock need literal block notation to avoid it become
+            # paragraph.
+            if isinstance(node, LITERAL_TYPE_NODES):
+                msgstr = '::\n\n' + indent(msgstr, ' '*3)
+
             patch = new_document(source, settings)
             CustomLocaleReporter(node.source, node.line).set_reporter(patch)
             parser.parse(msgstr, patch)
@@ -306,13 +380,16 @@ class Locale(Transform):
             except IndexError:  # empty node
                 pass
             # XXX doctest and other block markup
-            if not isinstance(patch, nodes.paragraph):
-                continue # skip for now
+            if not isinstance(
+                    patch,
+                    (nodes.paragraph,) + LITERAL_TYPE_NODES + IMAGE_TYPE_NODES):
+                continue  # skip for now
 
             # auto-numbered foot note reference should use original 'ids'.
             def is_autonumber_footnote_ref(node):
                 return isinstance(node, nodes.footnote_reference) and \
                     node.get('auto') == 1
+
             def list_replace_or_append(lst, old, new):
                 if old in lst:
                     lst[lst.index(old)] = new
@@ -337,7 +414,7 @@ class Locale(Transform):
                 for id in new['ids']:
                     self.document.ids[id] = new
                 list_replace_or_append(
-                        self.document.autofootnote_refs, old, new)
+                    self.document.autofootnote_refs, old, new)
                 if refname:
                     list_replace_or_append(
                         self.document.footnote_refs.setdefault(refname, []),
@@ -402,6 +479,7 @@ class Locale(Transform):
             if len(old_refs) != len(new_refs):
                 env.warn_node('inconsistent term references in '
                               'translated message', node)
+
             def get_ref_key(node):
                 case = node["refdomain"], node["reftype"]
                 if case == ('std', 'term'):
@@ -415,34 +493,49 @@ class Locale(Transform):
             for old in old_refs:
                 key = get_ref_key(old)
                 if key:
-                    xref_reftarget_map[key] = old["reftarget"]
+                    xref_reftarget_map[key] = old.attributes
             for new in new_refs:
                 key = get_ref_key(new)
-                if key in xref_reftarget_map:
-                    new['reftarget'] = xref_reftarget_map[key]
+                # Copy attributes to keep original node behavior. Especially
+                # copying 'reftarget', 'py:module', 'py:class' are needed.
+                for k, v in xref_reftarget_map.get(key, {}).items():
+                    # Note: This implementation overwrite all attributes.
+                    # if some attributes `k` should not be overwritten,
+                    # you should provide exclude list as:
+                    # `if k not in EXCLUDE_LIST: new[k] = v`
+                    new[k] = v
 
             # update leaves
             for child in patch.children:
                 child.parent = node
             node.children = patch.children
+
+            # for highlighting that expects .rawsource and .astext() are same.
+            if isinstance(node, LITERAL_TYPE_NODES):
+                node.rawsource = node.astext()
+
+            if isinstance(node, IMAGE_TYPE_NODES):
+                node.update_all_atts(patch)
+
             node['translated'] = True
 
-        # Extract and translate messages for index entries.
-        for node, entries in traverse_translatable_index(self.document):
-            new_entries = []
-            for type, msg, tid, main in entries:
-                msg_parts = split_index_msg(type, msg)
-                msgstr_parts = []
-                for part in msg_parts:
-                    msgstr = catalog.gettext(part)
-                    if not msgstr:
-                        msgstr = part
-                    msgstr_parts.append(msgstr)
+        if 'index' in env.config.gettext_additional_targets:
+            # Extract and translate messages for index entries.
+            for node, entries in traverse_translatable_index(self.document):
+                new_entries = []
+                for type, msg, tid, main in entries:
+                    msg_parts = split_index_msg(type, msg)
+                    msgstr_parts = []
+                    for part in msg_parts:
+                        msgstr = catalog.gettext(part)
+                        if not msgstr:
+                            msgstr = part
+                        msgstr_parts.append(msgstr)
 
-                new_entries.append((type, ';'.join(msgstr_parts), tid, main))
+                    new_entries.append((type, ';'.join(msgstr_parts), tid, main))
 
-            node['raw_entries'] = entries
-            node['entries'] = new_entries
+                node['raw_entries'] = entries
+                node['entries'] = new_entries
 
 
 class RemoveTranslatableInline(Transform):
