@@ -5,14 +5,13 @@
 
     Utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
 import re
 import sys
-import shutil
 import fnmatch
 import tempfile
 import posixpath
@@ -22,22 +21,27 @@ from os import path
 from codecs import open, BOM_UTF8
 from collections import deque
 
+from six import iteritems, text_type, binary_type
+from six.moves import range
 import docutils
 from docutils.utils import relative_path
 
 import jinja2
 
 import sphinx
-from sphinx.errors import PycodeError
-from sphinx.util.pycompat import bytes
+from sphinx.errors import PycodeError, SphinxParallelError, ExtensionError
+from sphinx.util.console import strip_colors
+from sphinx.util.osutil import fs_encoding
 
 # import other utilities; partly for backwards compatibility, so don't
 # prune unused ones indiscriminately
-from sphinx.util.osutil import SEP, os_path, relative_uri, ensuredir, walk, \
-     mtimes_of_files, movefile, copyfile, copytimes, make_filename, ustrftime
-from sphinx.util.nodes import nested_parse_with_titles, split_explicit_title, \
-     explicit_title_re, caption_ref_re
-from sphinx.util.matching import patfilter
+from sphinx.util.osutil import (  # noqa
+    SEP, os_path, relative_uri, ensuredir, walk, mtimes_of_files, movefile,
+    copyfile, copytimes, make_filename, ustrftime)
+from sphinx.util.nodes import (   # noqa
+    nested_parse_with_titles, split_explicit_title, explicit_title_re,
+    caption_ref_re)
+from sphinx.util.matching import patfilter  # noqa
 
 # Generally useful regular expressions.
 ws_re = re.compile(r'\s+')
@@ -54,7 +58,7 @@ def docname_join(basedocname, docname):
 def path_stabilize(filepath):
     "normalize path separater and unicode string"
     newpath = filepath.replace(os.path.sep, SEP)
-    if isinstance(newpath, unicode):
+    if isinstance(newpath, text_type):
         newpath = unicodedata.normalize('NFC', newpath)
     return newpath
 
@@ -85,17 +89,18 @@ def get_matching_files(dirname, exclude_matchers=()):
             yield filename
 
 
-def get_matching_docs(dirname, suffix, exclude_matchers=()):
-    """Get all file names (without suffix) matching a suffix in a directory,
+def get_matching_docs(dirname, suffixes, exclude_matchers=()):
+    """Get all file names (without suffixes) matching a suffix in a directory,
     recursively.
 
     Exclude files and dirs matching a pattern in *exclude_patterns*.
     """
-    suffixpattern = '*' + suffix
+    suffixpatterns = ['*' + s for s in suffixes]
     for filename in get_matching_files(dirname, exclude_matchers):
-        if not fnmatch.fnmatch(filename, suffixpattern):
-            continue
-        yield filename[:-len(suffix)]
+        for suffixpattern in suffixpatterns:
+            if fnmatch.fnmatch(filename, suffixpattern):
+                yield filename[:-len(suffixpattern)+1]
+                break
 
 
 class FilenameUniqDict(dict):
@@ -122,12 +127,17 @@ class FilenameUniqDict(dict):
         return uniquename
 
     def purge_doc(self, docname):
-        for filename, (docs, unique) in self.items():
+        for filename, (docs, unique) in list(self.items()):
             docs.discard(docname)
             if not docs:
                 del self[filename]
                 self._existing.discard(unique)
 
+    def merge_other(self, docnames, other):
+        for filename, (docs, unique) in other.items():
+            for doc in docs & docnames:
+                self.add_file(doc, filename)
+
     def __getstate__(self):
         return self._existing
 
@@ -142,7 +152,7 @@ def copy_static_entry(source, targetdir, builder, context={},
     Handles all possible cases of files, directories and subdirectories.
     """
     if exclude_matchers:
-        relpath = relative_path(builder.srcdir, source)
+        relpath = relative_path(path.join(builder.srcdir, 'dummy'), source)
         for matcher in exclude_matchers:
             if matcher(relpath):
                 return
@@ -173,28 +183,45 @@ def copy_static_entry(source, targetdir, builder, context={},
 
 _DEBUG_HEADER = '''\
 # Sphinx version: %s
-# Python version: %s
+# Python version: %s (%s)
 # Docutils version: %s %s
 # Jinja2 version: %s
+# Last messages:
+%s
 # Loaded extensions:
 '''
 
+
 def save_traceback(app):
     """Save the current exception's traceback in a temporary file."""
     import platform
-    exc = traceback.format_exc()
+    exc = sys.exc_info()[1]
+    if isinstance(exc, SphinxParallelError):
+        exc_format = '(Error in parallel process)\n' + exc.traceback
+    else:
+        exc_format = traceback.format_exc()
     fd, path = tempfile.mkstemp('.log', 'sphinx-err-')
+    last_msgs = ''
+    if app is not None:
+        last_msgs = '\n'.join(
+            '#   %s' % strip_colors(force_decode(s, 'utf-8')).strip()
+            for s in app.messagelog)
     os.write(fd, (_DEBUG_HEADER %
                   (sphinx.__version__,
                    platform.python_version(),
+                   platform.python_implementation(),
                    docutils.__version__, docutils.__version_details__,
-                   jinja2.__version__)).encode('utf-8'))
+                   jinja2.__version__,
+                   last_msgs)).encode('utf-8'))
     if app is not None:
-        for extname, extmod in app._extensions.iteritems():
-            os.write(fd, ('#   %s from %s\n' % (
-                extname, getattr(extmod, '__file__', 'unknown'))
-                ).encode('utf-8'))
-    os.write(fd, exc.encode('utf-8'))
+        for extname, extmod in iteritems(app._extensions):
+            modfile = getattr(extmod, '__file__', 'unknown')
+            if isinstance(modfile, bytes):
+                modfile = modfile.decode(fs_encoding, 'replace')
+            os.write(fd, ('#   %s (%s) from %s\n' % (
+                extname, app._extension_metadata[extname]['version'],
+                modfile)).encode('utf-8'))
+    os.write(fd, exc_format.encode('utf-8'))
     os.close(fd)
     return path
 
@@ -208,7 +235,7 @@ def get_module_source(modname):
     if modname not in sys.modules:
         try:
             __import__(modname)
-        except Exception, err:
+        except Exception as err:
             raise PycodeError('error importing %r' % modname, err)
     mod = sys.modules[modname]
     filename = getattr(mod, '__file__', None)
@@ -216,12 +243,12 @@ def get_module_source(modname):
     if loader and getattr(loader, 'get_filename', None):
         try:
             filename = loader.get_filename(modname)
-        except Exception, err:
+        except Exception as err:
             raise PycodeError('error getting filename for %r' % filename, err)
     if filename is None and loader:
         try:
             return 'string', loader.get_source(modname)
-        except Exception, err:
+        except Exception as err:
             raise PycodeError('error getting source for %r' % modname, err)
     if filename is None:
         raise PycodeError('no source found for module %r' % modname)
@@ -238,9 +265,24 @@ def get_module_source(modname):
     return 'file', filename
 
 
+def get_full_modname(modname, attribute):
+    __import__(modname)
+    module = sys.modules[modname]
+
+    # Allow an attribute to have multiple parts and incidentially allow
+    # repeated .s in the attribute.
+    value = module
+    for attr in attribute.split('.'):
+        if attr:
+            value = getattr(value, attr)
+
+    return getattr(value, '__module__', None)
+
+
 # a regex to recognize coding cookies
 _coding_re = re.compile(r'coding[:=]\s*([-\w.]+)')
 
+
 def detect_encoding(readline):
     """Like tokenize.detect_encoding() from Py3k, but a bit simplified."""
 
@@ -328,7 +370,7 @@ def parselinenos(spec, total):
             else:
                 start = (begend[0] == '') and 0 or int(begend[0])-1
                 end = (begend[1] == '') and total or int(begend[1])
-                items.extend(xrange(start, end))
+                items.extend(range(start, end))
         except Exception:
             raise ValueError('invalid line number spec: %r' % spec)
     return items
@@ -336,7 +378,7 @@ def parselinenos(spec, total):
 
 def force_decode(string, encoding):
     """Forcibly get a unicode string out of a bytestring."""
-    if isinstance(string, bytes):
+    if isinstance(string, binary_type):
         try:
             if encoding:
                 string = string.decode(encoding)
@@ -352,8 +394,10 @@ def force_decode(string, encoding):
 class attrdict(dict):
     def __getattr__(self, key):
         return self[key]
+
     def __setattr__(self, key, val):
         self[key] = val
+
     def __delattr__(self, key):
         del self[key]
 
@@ -368,7 +412,7 @@ def rpartition(s, t):
 
 def split_into(n, type, value):
     """Split an index entry into a given number of parts at semicolons."""
-    parts = map(lambda x: x.strip(), value.split(';', n-1))
+    parts = [x.strip() for x in value.split(';', n-1)]
     if sum(1 for part in parts if part) < n:
         raise ValueError('invalid %s index entry %r' % (type, value))
     return parts
@@ -400,7 +444,7 @@ def split_index_msg(type, value):
 def format_exception_cut_frames(x=1):
     """Format an exception with traceback, but only the last x frames."""
     typ, val, tb = sys.exc_info()
-    #res = ['Traceback (most recent call last):\n']
+    # res = ['Traceback (most recent call last):\n']
     res = []
     tbres = traceback.format_tb(tb)
     res += tbres[-x:]
@@ -420,11 +464,13 @@ class PeekableIterator(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         """Return the next item from the iterator."""
         if self.remaining:
             return self.remaining.popleft()
-        return self._iterator.next()
+        return next(self._iterator)
+
+    next = __next__  # Python 2 compatibility
 
     def push(self, item):
         """Push the `item` on the internal stack, it will be returned on the
@@ -434,6 +480,46 @@ class PeekableIterator(object):
 
     def peek(self):
         """Return the next item without changing the state of the iterator."""
-        item = self.next()
+        item = next(self)
         self.push(item)
         return item
+
+
+def get_figtype(node):
+    """Return figtype for given node."""
+    def has_child(node, cls):
+        return any(isinstance(child, cls) for child in node)
+
+    from docutils import nodes
+    if isinstance(node, nodes.figure):
+        return 'figure'
+    elif isinstance(node, nodes.image) and isinstance(node.parent, nodes.figure):
+        # bare image node is not supported because it doesn't have caption and
+        # no-caption-target isn't a numbered figure.
+        return 'figure'
+    elif isinstance(node, nodes.table):
+        return 'table'
+    elif isinstance(node, nodes.container):
+        if has_child(node, nodes.literal_block):
+            return 'code-block'
+
+    return None
+
+
+def import_object(objname, source=None):
+    try:
+        module, name = objname.rsplit('.', 1)
+    except ValueError as err:
+        raise ExtensionError('Invalid full object name %s' % objname +
+                             (source and ' (needed for %s)' % source or ''),
+                             err)
+    try:
+        return getattr(__import__(module, None, None, [name]), name)
+    except ImportError as err:
+        raise ExtensionError('Could not import %s' % module +
+                             (source and ' (needed for %s)' % source or ''),
+                             err)
+    except AttributeError as err:
+        raise ExtensionError('Could not find %s' % objname +
+                             (source and ' (needed for %s)' % source or ''),
+                             err)
